/*
 * fstest.c - user operations to test vnode memory management.
 * A slightly modified version of Terry's fstest.c for testing
 * namei path buffer management.
 */
/*
 * Copyright (c) 1997, Terrence Lambert
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Terrence Lambert.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <fcntl.h>

#include "testvn.h"


/*
 * manifest terminal component names relative to "dir".
 */
#define	nonexistant_file	"nonexistant_file"
#define	nonexistant_dir		"nonexistant_dir"
#define	existing_file		"existing_file"
#define	existing_dir		"existing_dir"
#define	renamed_file		"renamed_file"
#define	renamed_dir		"renamed_dir"
#define	new_file		"new_file"
#define	new_dir			"new_dir"
#define	new_device		"new_device"
#define	new_link		"new_link"
#define	new_symlink		"new_symlink"

/*
 * Globals...
 */
static char	*progname;		/* name of this program (av[ 0])*/
static char	*dir;			/* name of test directory*/
static char	ne_file[ 1024];		/* path to "nonexistant file"*/
#define	ne_dir	ne_file			/* nonexistant -- doesn't matter*/
static char	ex_file[ 1024];		/* path to "existing file"*/
static char	ex_dir[ 1024];		/* path to "existing dir"*/
static char	rn_file[ 1024];		/* path to "renamed file"*/
static char	rn_dir[ 1024];		/* path to "renamed dir"*/
static char	nw_file[ 1024];		/* path to "new file"*/
static char	nw_dir[ 1024];		/* path to "new dir"*/
static char	nw_device[ 1024];	/* path to "new device"*/
static char	nw_link[ 1024];		/* path to "new link"*/
static char	nw_symlink[ 1024];	/* path to "new symlink"*/



/*
 * Special: teardown test environment (should not error)
 */
void
usr_teardown()
{
	struct stat	sb;


	/*
	 * "existing" file there?
	 */
	if( stat( ex_file, &sb) == 0) {
		/*
		 * yes, remove.
		 */
		unlink( ex_file);
	}

	/*
	 * "existing" dir there?
	 */
	if( stat( ex_dir, &sb) == 0) {
		/*
		 * yes, remove.
		 */
		rmdir( ex_dir);
	}

	/*
	 * "renamed" file there?
	 */
	if( stat( rn_file, &sb) == 0) {
		/*
		 * yes, remove.
		 */
		unlink( rn_file);
	}

	/*
	 * "renamed" dir there?
	 */
	if( stat( rn_dir, &sb) == 0) {
		/*
		 * yes, remove.
		 */
		rmdir( rn_dir);
	}

	/*
	 * "new" file there?
	 */
	if( stat( nw_file, &sb) == 0) {
		/*
		 * yes, remove.
		 */
		unlink( nw_file);
	}

	/*
	 * "new" dir there?
	 */
	if( stat( nw_dir, &sb) == 0) {
		/*
		 * yes, remove.
		 */
		rmdir( nw_dir);
	}

	/*
	 * "new" device there?
	 */
	if( stat( nw_device, &sb) == 0) {
		/*
		 * yes, remove.
		 */
		unlink( nw_device);
	}

	/*
	 * "new" link there?
	 */
	if( stat( nw_link, &sb) == 0) {
		/*
		 * yes, remove.
		 */
		unlink( nw_link);
	}

	/*
	 * "new" symlink there?
	 */
	if( lstat( nw_symlink, &sb) == 0) {
		/*
		 * yes, remove.
		 */
		unlink( nw_symlink);
	}

}


/*
 * Setup test environment
 *
 * Argument interpretation is left up to the user.
 *
 * Returns:	0	Success
 *		1	Error
 */
int
usr_setup( ac, av)
int	ac;
char	*av[];
{
	int		rv = 1;		/* default: error*/
	struct stat	sb;
	int		fd;

	progname = av[ 0];

	/*
	 * verify usage
	 */
	if( ac != 2) {
		fprintf( stderr, "usage: %s <test directory>\n", progname);
		goto err;
	}

	/*
	 * Verify access to "test directory"
	 */
	dir = av[ 1];
	if( stat( dir, &sb) != 0) {
		fprintf( stderr, "%s: %s: no such directory\n", progname, dir);
		goto err;
	}


	/*
	 * Build the file paths
	 */
	sprintf( ne_file, "%s/%s", dir, nonexistant_file);
	sprintf( ex_file, "%s/%s", dir, existing_file);
	sprintf( ex_dir, "%s/%s", dir, existing_dir);
	sprintf( rn_file, "%s/%s", dir, renamed_file);
	sprintf( rn_dir, "%s/%s", dir, renamed_dir);
	sprintf( nw_file, "%s/%s", dir, new_file);
	sprintf( nw_dir, "%s/%s", dir, new_dir);
	sprintf( nw_device, "%s/%s", dir, new_device);
	sprintf( nw_link, "%s/%s", dir, new_link);
	sprintf( nw_symlink, "%s/%s", dir, new_symlink);


	/*
	 * Create the "existing" file
	 */
	if( ( fd = open( ex_file, O_CREAT|O_TRUNC|O_WRONLY, 0666)) == -1) {
		fprintf( stderr, "%s: %s: can't create\n", progname, ex_file);
		goto err_1;
	}
	close( fd);


	/*
	 * Create the "existing" directory
	 */
	if( mkdir( ex_dir, 0666)) {
		fprintf( stderr, "%s: %s: can't create\n", progname, ex_dir);
		goto err_1;
	}



	rv = 0;		/* success*/

	goto success;

err_1:
	/*
	 * cheat: use tolerant tear-down to back out state
	 */
	usr_teardown();
err:

success:
	return( rv);
}



/*
 * Open Existing File test
 *
 * open then close an existing file.
 */
static void
open_existing_file()
{
	int	fd;

	fd = open( ex_file, O_RDONLY, 0);
	close( fd);
}


/*
 * Open Nonexistant File test
 *
 * open a nonexistant file.
 *
 * Note: ENOENT expected
 */
static void
open_nonexistant_file()
{
	open( ne_file, O_RDONLY, 0);
}


/*
 * Open Existing Dir test
 *
 * open then close an existing dir.
 */
static void
open_existing_dir()
{
	int	fd;

	/*
	 * R/O open of dir is allowed under POSIX if directories are
	 * implemented using files.
	 */
	fd = open( ex_file, O_RDONLY, 0);
	close( fd);
}


/*
 * Stat Existing File test
 *
 * stat an existing file.
 */
static void
stat_existing_file()
{
	struct stat	sb;

	stat( ex_file, &sb);
}


/*
 * Stat Nonexistant File test
 *
 * stat a nonexistant file.
 *
 * Note: ENOENT expected
 */
static void
stat_nonexistant_file()
{
	struct stat	sb;

	stat( ne_file, &sb);
}


/*
 * Stat Existing Dir test
 *
 * stat an existing dir.
 */
static void
stat_existing_dir()
{
	struct stat	sb;

	stat( ex_dir, &sb);
}


/*
 * Rename Existing File To Dir test
 *
 * rename an existing file to an existing dir.
 *
 * Note: EISDIR expected
 */
static void
rename_existing_file_to_dir()
{
	rename( ex_file, ex_dir);
}


/*
 * Rename Existing Dir To File test
 *
 * rename an existing dir to an existing file.
 *
 * Note: ENOTDIR expected
 */
static void
rename_existing_dir_to_file()
{
	rename( ex_dir, ex_file);
}


/*
 * Rename Nonexistant File test
 *
 * rename an nonexistant file to an exiting file.
 *
 * Note: ENOENT expected
 */
static void
rename_nonexistant_file()
{
	rename( ne_file, ex_file);
}


/*
 * Rename Nonexistant Dir test
 *
 * rename a nonexistant dir to an existing dir.
 *
 * Note: ENOENT expected
 */
static void
rename_nonexistant_dir()
{
	rename( ne_dir, ex_dir);
}


/*
 * Rename Existing File test
 *
 * rename an existing file.
 */
static void
rename_existing_file()
{
	rename( ex_file, rn_file);
}


/*
 * Rename Existing Dir test
 *
 * rename  an existing dir.
 */
static void
rename_existing_dir()
{
	rename( ex_dir, rn_dir);
}


/*
 * Rmdir Base Dir test
 *
 * rmdir the base test directory
 *
 * Note: ENOTEMPTY expected
 */
static void
rmdir_base_dir()
{
	rmdir( dir);
}


/*
 * Rmdir Nonexistant Dir test
 *
 * rmdir a non-existant dir.
 *
 * Note: ENOENT expected
 */
static void
rmdir_nonexistant_dir()
{
	rmdir( ne_dir);
}


/*
 * Unlink Nonexistant File test
 *
 * unlink a non-existant file.
 *
 * Note: ENOENT expected
 */
static void
unlink_nonexistant_file()
{
	unlink( ne_file);
}


/*
 * Rmdir Renamed File test
 *
 * rmdir a renamed (existing) file.
 *
 * Note: EPERM expected
 */
static void
rmdir_renamed_file()
{
	rmdir( rn_file);
}


/*
 * Unlink Renamed Dir test
 *
 * unlink a renamed (existing) dir.
 *
 * Note: EPERM expected
 */
static void
unlink_renamed_dir()
{
	unlink( rn_dir);
}


/*
 * Unlink Renamed File test
 *
 * unlink a renamed (existing) file.
 */
static void
unlink_renamed_file()
{
	unlink( rn_file);
}


/*
 * Rmdir Renamed Dir test
 *
 * rmdir a renamed (existing) dir.
 */
static void
rmdir_renamed_dir()
{
	rmdir( rn_dir);
}


/*
 * Create New File test
 *
 * create a new file.
 */
static void
create_new_file()
{
	int	fd;

	fd = open( nw_file, O_CREAT|O_TRUNC|O_WRONLY, 0666);

	close( fd);
}


/*
 * Create New Dir test
 *
 * create a new dir.
 */
static void
create_new_dir()
{
	mkdir( nw_dir, 0666);
}


/*
 * Create New Device test
 *
 * create a new device.
 */
static void
create_new_device()
{
	dev_t dev;

	dev = makedev(0, 0);
	mknod( nw_device, 0666 | S_IFBLK, dev);
}


/*
 * Create New Link test
 *
 * create a new link.
 */
static void
create_new_link()
{
	link( nw_file, nw_link);
}


/*
 * Create New Symlink test
 *
 * create a new symlink.
 */
static void
create_new_symlink()
{
	symlink( new_file, nw_symlink);
}

/*
 *************************************************************************
 * Agregate of all tests, above...
 *************************************************************************
 */
struct testvn usr_tests[] = {
	{	"open_existing_file",
		open_existing_file,
		0,0,0
	},
	{	"open_nonexistant_file",
		open_nonexistant_file,
		0,0,0
	},
	{	"open_existing_file",
		open_existing_file,
		0,0,0
	},
	{	"open_existing_dir",
		open_existing_dir,
		0,0,0
	},
	{	"stat_existing_file",
		stat_existing_file,
		0,0,0
	},
	{	"stat_nonexistant_file",
		stat_nonexistant_file,
		0,0,0
	},
	{	"stat_existing_dir",
		stat_existing_dir,
		0,0,0
	},
	{	"rename_existing_file_to_dir",
		rename_existing_file_to_dir,
		0,0,0
	},
	{	"rename_existing_dir_to_file",
		rename_existing_dir_to_file,
		0,0,0
	},
	{	"rename_nonexistant_file",
		rename_nonexistant_file,
		0,0,0
	},
	{	"rename_nonexistant_dir",
		rename_nonexistant_dir,
		0,0,0
	},
	{	"rename_existing_file",
		rename_existing_file,
		0,0,0
	},
	{	"rename_existing_dir",
		rename_existing_dir,
		0,0,0
	},
	{	"rmdir_base_dir",
		rmdir_base_dir,
		0,0,0
	},
	{	"unlink_nonexistant_file",
		unlink_nonexistant_file,
		0,0,0
	},
	{	"rmdir_nonexistant_dir",
		rmdir_nonexistant_dir,
		0,0,0
	},
	{	"rmdir_renamed_file",
		rmdir_renamed_file,
		0,0,0
	},
	{	"unlink_renamed_dir",
		unlink_renamed_dir,
		0,0,0
	},
	{	"unlink_renamed_file",
		unlink_renamed_file,
		0,0,0
	},
	{	"rmdir_renamed_dir",
		rmdir_renamed_dir,
		0,0,0
	},
	{	"create_new_file",
		create_new_file,
		0,0,0
	},
	{	"create_new_dir",
		create_new_dir,
		0,0,0
	},
	{	"create_new_device",
		create_new_device,
		0,0,0
	},
	{	"create_new_link",
		create_new_link,
		0,0,0
	},
	{	"create_new_symlink",
		create_new_symlink,
		0,0,0
	},
	{ NULL }
};


/*
 * EOF: This file has not been truncated.
 */
